package com.logicbig.example;

import com.logicbig.uicommon.menu.MenuAction;
import com.logicbig.uicommon.menu.MenuBuilder;
import com.logicbig.uicommon.tree.JTreeUtil;
import javax.swing.*;
import java.awt.*;
import java.util.Arrays;
import java.util.List;

public class MenuBuilderExampleMain {
    public static void main(String[] args) {
        JTree tree = createJTree();
        JTable table = createJTable();
        MenuBuilder menuBuilder = buildMenu();
        menuBuilder.buildPopupMenu(tree);
        //in real scenario we will create different menu for each component
        menuBuilder.buildPopupMenu(table);
        JMenu menu = menuBuilder.buildMenu("Main menu");
        JMenuBar jMenuBar = new JMenuBar();
        jMenuBar.add(menu);

        JFrame frame = createFrame();
        frame.setJMenuBar(jMenuBar);
        frame.add(new JScrollPane(tree), BorderLayout.WEST);
        frame.add(new JScrollPane(table), BorderLayout.CENTER);
        frame.setLocationRelativeTo(null);
        frame.setVisible(true);
    }

    private static MenuBuilder buildMenu() {
        //in real scenario separate actions should be created for each menu
        MenuAction nodeAction = createNodeAction();
        return MenuBuilder.init()
                          .menu("1", nodeAction)
                          .menu("2", nodeAction)
                          .parent("3", nodeAction)
                          .menu("4", nodeAction)
                          .menu("5", nodeAction)
                          //nested parent
                          .parent("5a", nodeAction)
                          .menu("5a1", nodeAction)
                          .menu("5a2", nodeAction)
                          .parentEnd()
                          .menu("5b", nodeAction)
                          .parentEnd()
                          .menu("6", nodeAction);
    }

    private static MenuAction createNodeAction() {
        return new MenuAction() {
            @Override
            public void perform(String command, List<?> selection) {
                System.out.println("menu action invoked: " + command);
                System.out.println("selections: ");
                for (Object o : selection) {
                    if (o instanceof Object[]) {//table row
                        System.out.println(Arrays.toString((Object[]) o));
                    } else {
                        System.out.println(o);
                    }
                }
            }

            @Override
            public boolean shouldEnable(String command, List<?> selection) {
                System.out.println("shouldEnable invoked: " + command);
                //just disable some randomly,
                // in real scenario we will disable based on selection, command and some other external dynamic params
                return Math.random() < 0.6;
            }
        };
    }

    private static JTable createJTable() {
        return new JTable(new Object[][]{
                new Object[]{1, 2, 3},
                new Object[]{4, 5, 6},
                new Object[]{7, 8, 9}},
                new String[]{"one", "two", "three"});
    }

    private static JTree createJTree() {
        JTree tree = new JTree();//using default tree
        JTreeUtil.setTreeExpandedState(tree, true);
        return tree;
    }

    private static JFrame createFrame() {
        JFrame frame = new JFrame("Menu Builder Example");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setSize(new Dimension(600, 400));
        return frame;
    }
}